from constants import *

#
# Bids are placed on a switch, to make it connect
# specified lines.
#
# Bids are increasing in a bid-run
#

class Bid:
	
	def __init__(self, gamestate, switch):
		self.gamestate = gamestate
		self.switch = switch
		self.topplayer = False	
			# otherwise reference to Player with highest bid
		self.topstate = 0;
		self.topbid = 0;
			# and the state she prefers, the money she offers
			
		self.player_states = [False for i in range(0, MAXPLAYERS)]
			# contains 'switch state' pairs for each player
			# or False if player did not bid yet
		self.player_bids = [0 for i in range(0, MAXPLAYERS)]
			# Stores how much each player bidded in a round
			# this is used to give back money for lsoers
		self.delta_bid = DELTA_BID_START
			# The amount that has to be next added to the bid
		self.frozen = False
			# Tells if the bid is closed


	
	def bidForState(self, player, state, money):
		# if bid is frozen then return
		if self.frozen:
			return
		
		# check if player has enough action points
		# if she hasn't, nothing really happens
		amount = self.delta_bid + self.topbid - self.player_bids[player.id]
		
		# Can only take the switch at the desired price, not at a higher one
		if money != amount:
			return
		
		if player.action_points >= amount:
						
			# register action points
			player.action_points -= amount
			
			self.topbid += self.delta_bid
			self.player_bids[player.id] += amount
						
			# update top bidder status
			self.topplayer = player
			
			# set new state
			self.player_states[player.id] = state
			self.topstate = state
			
			
			# increase delta bid amount
			self.delta_bid += DELTA_BID_INCREASE
			
			# register change
			self.switch.changed = True



	def freeze(self):
		self.frozen = True
		
		# if noone bidded, do nothing
		if self.topplayer is False:
			return
			
		# iterate bidders
		pl = 0
		for bid_rec in self.player_bids:
			if (pl != self.topplayer.id) and bid_rec:
				# give losers back some money	
				self.gamestate.players[pl].action_points += bid_rec * BID_REFUND_RATIO

			# Count index, too
			pl += 1

		# reset bids of players
		self.player_bids = [0 for i in range(0, len(self.gamestate.players))]
		self.player_states = [False for i in range(0, len(self.gamestate.players))]
		
		# reset switch bid
		self.delta_bid = DELTA_BID_START
		self.topbid = 0


	def unfreeze(self):
		self.frozen = False
		self.topplayer = False
