################################################
## This will get attached to the Gamestate class
################################################

from constants import *
from string import split, strip
import os

from segment import Segment
from arc import Arc
from line import Line
from switch import GSwitch

#
#File format is:
#
#	#lines
#	#line0_segs [coal_begin coal_end]
#	s x1 y1 x2 y1
#	a center_x center_y radius start_phi end_phi
#	...
#	...
#	#switches
#	#switch0 states
#	line1id 1|-1 line2id 1|-1
#	...
#	...
#	specline1_id specline2_id specline3_id specline4_id
#	#coal_pos
#	coal_x coal_y line_id

SCALE = 1

# Loads railway track data from file
def loadMapFile(self, filename, resources):
	self.mapname = filename

	self.lines = []
	self.switches = []

	f = file(os.path.join(MAP_PATH, filename), 'r')
 
	def noEmpties(lst):
		try:
			while True:
				lst.remove('')
		except ValueError:
			return
	
	# Load lines
	numlines = int( strip( f.readline() ) )
	
	for line_idx in range(0, numlines):
		
		# Load line info
		lineinfo = split( strip( f.readline() ), " " )
		noEmpties(lineinfo)
		
		numsegs = int( lineinfo[0] )
		coal = len(lineinfo) > 1
		if coal:
			# Coal data is present
			coal = (float(lineinfo[1]), float(lineinfo[2]))
		
		# Create line
		line = Line(line_idx, coal)
		
		# Load segments
		for seg_idx in range(0, numsegs):
			
			# Load the data
			data = split( strip( f.readline() ), " " )
			noEmpties(data)
			
			if data[0] == "s":
				# Segment
				line.addSegment( Segment( (SCALE*float(data[1]), SCALE*float(data[2])), (SCALE*float(data[3]), SCALE*float(data[4])) ) )
			elif data[0] == "a":
				# Arc
				line.addSegment( Arc( (SCALE*float(data[1]), SCALE*float(data[2])), SCALE*float(data[3]), float(data[4]), float(data[5]) ) )
			else:
				raise Exception("File format error")
		
		# Line complete, add to list
		self.addLine(line)
	
	# Load switches
	numswitches = int( strip( f.readline() ) )
	
	for sw_idx in range(0, numswitches):
	
		# Create switch
		sw = GSwitch(sw_idx, self, resources)
		# Load states
		numstates = int( strip( f.readline() ) )
		
		for st_idx in range(0, numstates):
			data = split( strip( f.readline() ), " " )
			noEmpties(data)
			# Add state
			sw.addState( ((self.lines[int(data[0])], int(data[1])), (self.lines[int(data[2])], int(data[3]))) )
		
		# Switch complete, add to list
		self.addSwitch(sw)
	
	# Load spawning lines
	data = split( strip( f.readline() ), " " )
	noEmpties(data)
	for ln in data:
		self.spawnlines.append( self.lines[ int(ln) ] )
		
	# Read number of coal bases
	numcoalbases = int( strip( f.readline() ) )
	
	for cb_idx in range(0, numcoalbases):
		data = split( strip( f.readline() ), " " )
		noEmpties(data)
		self.coalbases[self.lines[ int(data[2]) ]] = (SCALE*float(data[0]), SCALE*float(data[1]))
		
		
	# Loading complete
	f.close()
	print "Loaded", filename
	
	
	

