###############################################
## These will get attached to the Gamestate class
################################################

from constants import *
from struct import *
from netformats import *

def bidTopplayer2byte(topplayer):
	if topplayer is False:
		return 255
	else:
		return topplayer.id

	
def createUpdatePackage(self, pkg_count, fullSync = False):
	pl_data = ""	# Players data string
	sw_data = "" # Switch data string
	
	for pl in self.players:
		tr = pl.train
		head = tr.head
		tail = tr.tail
		pl_data += pack(playerFormat, pl.id, pl.score, head[0].id, head[1], head[2], tail[0].id, tail[1], tail[2], tr.cars, tr.movesForward, tr.v, tr.force, tr.maxforce, tr.isBraking, tr.doSpeedup, tr.doBrake)
	
	sw_count = 0
	for sw in self.switches:
		if fullSync or sw.changed:
			sw_count += 1
			sw.changed = False
			bid = sw.bid
			sw_data += pack(switchFormat, sw.id, sw.state, sw.trainPassing, sw.trainStillPassing, bidTopplayer2byte(bid.topplayer), bid.topstate, bid.topbid, bid.delta_bid, bid.frozen)
	
	pkg = pack(updateHeaderFormat, NET_UPDATE_VERSION, pkg_count, len(self.players), sw_count)
	pkg += pl_data
	pkg += sw_data
	
	return pkg

	
	
def createPersonalPackage(self, player, updatePackage):
	# Append personal data
	return updatePackage + pack(privateFormat, player.action_points, player.train.coal) 
	
	
		
def syncToPersonalPackage(self, personalPackage, player, packageCount):
	# Extract header info
	update_version, pkg_count, pl_count, sw_count = unpack(updateHeaderFormat, personalPackage[:updateHeaderSize])
	
	if pkg_count <= packageCount:
		# Old-timer package
		return packageCount
	
	# Strip header
	personalPackage = personalPackage[updateHeaderSize:]
		
	for i in range(0, pl_count):
		# Unpack
		pl_id, pl_score, head_line, head_pos, head_end, tail_line, tail_pos, tail_end, cars, movesForward, v, force, maxforce, isBraking, doSpeedup, doBrake = unpack(playerFormat, personalPackage[:playerSize])
		
		# Store
		pl = self.players[pl_id]
		tr = pl.train
		pl.score = pl_score
		
		tr_head_line = self.lines[head_line]
		tr_tail_line = self.lines[tail_line]
		
		tr.head = (tr_head_line, head_pos, head_end)
		tr.tail = (tr_tail_line, tail_pos, tail_end)
		
		tr.cars, tr.movesForward, tr.v, tr.force, tr.maxforce, tr.isBraking, tr.doSpeedup, tr.doBrake = cars, bool(movesForward), v, force, maxforce, bool(isBraking), bool(doSpeedup), bool(doBrake)
		
		# Strip player
		personalPackage = personalPackage[playerSize:]
		
	for i in range(0, sw_count):
		# Unpack
		sw_id, sw_state, sw_trainPassing, sw_trainStillPassing, bid_topplayer, bid_topstate, bid_topbid, bid_delta_bid, bid_frozen = unpack(switchFormat, personalPackage[:switchSize])
		
		# Store
		sw = self.switches[sw_id]
		bid = sw.bid
		sw.state, sw.trainPassing, sw.trainStillPassing = sw_state, bool(sw_trainPassing), bool(sw_trainStillPassing)
		
		if bid_topplayer < 255:
			bid.topplayer = self.players[bid_topplayer]
		else:
			bid.topplayer = False
		
		#
		# Check here:
		# If we detect our own player as top bidder, register
		# to that switch's bid.player_bid
		# If we detect a switch to be in frozen state, clear
		# own player_bid to 0
		#
		if bid_frozen:
			sw.bid.player_bids[self.own_id] = 0
		elif bid.topplayer == self.ownplayer:
			sw.bid.player_bids[self.own_id] = bid_topbid
			
		bid.topstate, bid.topbid, bid.delta_bid, bid.frozen = bid_topstate, bid_topbid, bid_delta_bid, bid_frozen
		
		# Strip switch
		personalPackage = personalPackage[switchSize:]
	
	# Unpack personal data
	player.action_points, player.train.coal = unpack(privateFormat, personalPackage)
	
	return pkg_count
		