from pygame.image import tostring, load as loadImage, save, fromstring
from OpenGL.GL import *
from constants import WHITE, IMAGE_PATH
import os
#
# For loading an image file into an OpenGl texture
# and then rendering it as a quad
#
class Image:
	def __init__(self, filename = None, filter = GL_LINEAR, strdata = False, tex_size = False, render_size = False):
		# First load the image
		# It has to be RGBA8 formatted
		
		if not strdata:
			surf = loadImage( os.path.join(IMAGE_PATH, filename))
			
			if not render_size:
				self.w = surf.get_width()
				self.h = surf.get_height()
			else:
				self.w, self.h = render_size
			
			if not tex_size:
				tex_size = (self.w, self.h)
		
			# Pixel data
			data = tostring( surf , "RGBA", True)
			numbytes, format = 4, GL_RGBA
		else:
			self.w, self.h = render_size
			data = strdata
			numbytes = 3
			format = GL_RGB
			
		
		self.w2 = self.w / 2.0
		self.h2 = self.h / 2.0
		
		# Create texture
		self.texture = glGenTextures(1)
		glBindTexture(GL_TEXTURE_2D, self.texture)
		
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter)
		glTexImage2D(GL_TEXTURE_2D, 0, numbytes, tex_size[0], tex_size[1], 0, format, GL_UNSIGNED_BYTE, data)
		
		#glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_)
		
		
	def drawCentered(self, pos, color = WHITE, identity = True, rot = 0.0, alpha = False, scale = False):
		# Use texture
		glBindTexture(GL_TEXTURE_2D, self.texture)
		# Get in position
		if identity:
			glLoadIdentity()
				
		glTranslatef(pos[0], pos[1], 0.0)
		
		if rot:
			glRotatef(rot, 0.0, 0.0, 1.0)
		
		if scale:
			glScalef(scale, scale, scale)
			
		if alpha:
			glColor4f(1.0, 1.0, 1.0, alpha)
		else:
			glColor3ubv(color)
		glBegin(GL_QUADS)
		# Begin from top-right, CCW
		glTexCoord2f(1.0,1.0)
		glVertex2f(self.w2, self.h2)
		glTexCoord2f(0.0, 1.0)
		glVertex2f(-self.w2, self.h2)
		glTexCoord2f(0.0, 0.0)
		glVertex2f(-self.w2, -self.h2)
		glTexCoord2f(1.0, 0.0)
		glVertex2f(self.w2, -self.h2)
		glEnd()
		
	def draw(self, pos, color = WHITE, identity = True, rot = 0.0):
		# Use texture
		glBindTexture(GL_TEXTURE_2D, self.texture)
		# Get in position
		if identity:
			glLoadIdentity()
				
		glTranslatef(pos[0], pos[1], 0.0)
		
		if rot:
			glRotatef(rot, 0.0, 0.0, 1.0)
		
		glColor3ubv(color)
		glBegin(GL_QUADS)
		# Begin from top-right, CCW
		glTexCoord2f(1.0,1.0)
		glVertex2f(self.w, self.h)
		glTexCoord2f(0.0, 1.0)
		glVertex2f(0.0, self.h)
		glTexCoord2f(0.0, 0.0)
		glVertex2f(0.0, 0.0)
		glTexCoord2f(1.0, 0.0)
		glVertex2f(self.w, 0.0)
		glEnd()
	
	
	# Specially for the font textures
	def puttxt(self, pos, charpos, color4, scale = 2.0):
		glBindTexture(GL_TEXTURE_2D, self.texture)
		glLoadIdentity()
		glTranslatef(pos[0], pos[1], 0)
								
		tex_u1 = 16*charpos[0] / 128.0
		tex_v1 = 16*(7-charpos[1]) / 128.0
		tex_u2, tex_v2 = tex_u1 + 0.125, tex_v1 + 0.125
		
		glScalef(scale, scale, scale)
 		glColor4ubv(color4)
		glBegin(GL_QUADS)
		# Begin from top-right, CCW
		glTexCoord2f(tex_u2, tex_v2)
		glVertex2f(16, 16)
		
		glTexCoord2f(tex_u1, tex_v2)
		glVertex2f(0, 16)
		
		glTexCoord2f(tex_u1, tex_v1)
		glVertex2f(0, 0)
		
		glTexCoord2f(tex_u2, tex_v1)
		glVertex2f(16, 0)
		glEnd()
		
	
	def delete(self):
		pass
		#if self.texture is not False:
		#	print "Deleting texture"
		#	glDeleteTextures((self.texture))
		#	self.texture = False
		
	def __del__(self):
		self.delete()
		