from constants import *

#
# A Line lies between two Switches. It is build
# of Segments and Arcs
#
class Line:

	def __init__(self, id, coal = False):
		self.id = id		# unique id
		
		self.segments = []	# list of Segments and Arcs, in order from start
		self.length = 0.0	# total length
		self.coal = coal	# False or (from, to) scalar-pair
			
		self.start_switch = None	# These will be filled in, when
		self.end_switch = None		# the line gets added to switches
	
	
	# Switches will call this function to set refernce to themselves
	def setSwitch(self, sw, end):
		if end == 1:
			self.start_switch = sw
		else:
			self.end_switch = sw
	
	# Used to cache trains on the line
	# Trains call this while moving
	# def enterTrain(self, train):
	#	self.trains.append(train)
	
	# def leaveTrain(self, train):
	#	self.trains.remove(train)
		
	# To append a segment or arc
	def addSegment(self, seg):
		self.segments.append(seg)
		self.length += seg.length
	
	
	# Returns the distance of (s,d) to the switch sw
	def distance(self, s, d, sw):
		# Align direction so that it matches the end
		# where sw resides
		if (sw is self.start_switch) and (d == -1):
			s = self.length - s
		
		if (sw is self.end_switch) and (d == 1):
			s = self.length - s
		
		return s

	
	# 2D position of scalar s from direction d
	def getPos(self, s, d):
		
		if s > self.length:
			print 's is', s
			print 'length is', self.length
			raise Exception("Line not long enough")
	
		if d == -1:
			# From inside
			s = self.length - s
			d = 1
	
		len = 0.0
		for seg in self.segments:
			len += seg.length
			if s <= len:
				# The pos is the pos on this segment
				return seg.getPos(s - len + seg.length, 1)
			
		
		raise Exception("Position not found on line")
			

# The phisical position remains the same, but the direction is inverted
# This altered position is returned
def invertedLinePos(linepos):
	return ( linepos[LINE_REF_IDX], linepos[LINE_REF_IDX].length - linepos[LINE_POS_IDX], -linepos[LINE_END_IDX] )


#
# A tool for calculating directed line position from
# a directed line position and a distance
#
# If encounters a closed-state switch, it returns False
#
# Returns (line_ref, scalar_pos, 1|-1) 
#
# trainEnd signs, if the call is to determine
# the new position of a passing train-head or
# train-tail
def getLinePos(start, s, trainEnd = False):
	
	line, pos, end = start		# End signs the end, that we count from
	#print 'line', line.id, 'pos:', pos, 'End:', end, 'S:', s, 'length:', line.length
		
	while pos + s > line.length:
		### print 'iter'
		# Shift to the new line
		
		# Set next position to zero (beginning of line)
		# and decrease remaining length to go
		s -= line.length - pos
		pos = 0.0
		
		# Get the next line and end
		# Find them on the switch of (-end)
		###print line, line.start_switch, line.end_switch, line.id
		sw = line.start_switch
		###print sw
		if end == 1:	# No mistype here, end = -1 -> switch is at 1
			sw = line.end_switch
		
		#print "End, switch", end, sw.id
		#print "nextLine(line, -end)", (line.id, -end), "trainEnd:", trainEnd
		next_line = sw.getNextLine( (line, -end) , trainEnd )
		#print "done"
		
		# Is the switch open for us?
		if next_line is False:
			return False
		
		# Decode
		line, end = next_line
		#print "goes to (line, end)", line.id, end
	
	# Finally return actual line, position, end
	return (line, pos + s, end)


def get2DLinePos(start, s, invert = False):
	if invert:
		lp = invertedLinePos(start)
	else:
		lp = start
	
	lp = getLinePos(lp, s)
	if lp is False:
		print "BUG DETECTED!!!"
		return get2DLinePos(start, s-10.0, invert)
		# Give it a try...
	else:
		return lp[LINE_REF_IDX].getPos(lp[LINE_POS_IDX], lp[LINE_END_IDX])

# returns the sole position on the line, counted from the
# beginning of the line
def align1(linepos):
	if linepos[LINE_END_IDX] == 1:
		return linepos[1]
	else:
		return linepos[0].length - linepos[1]

	