from OpenGL.GL import *
from render import convertScreenPos

import gamestate, net as network, player

import pygame
from pygame.locals import *
import os

from constants import *
import config

MENU_QUIT = -10
MENU_NAME = 0
MENU_MAIN = 10
MENU_JOIN = 20
MENU_HOST = 30
MENU_JOINED = 40

white4 = (255, 255, 255, 255)

conditions = ("Win with score", "Win with number of carriages")
subconditions = ( ("200 points", "400 points", "600 points"), ("5 carriages")  )


class Menu:
	
	def __init__(self, resources):
		self.resources = resources
		
		# Selection index
		self.selectIdx = 0
		# Select pressed?
		self.select = False
		# Esc pressed?
		self.esc = False
		
		self.gamestate = False
		self.net = False
		self.maps = []
		
		# Menu states
		self.state = MENU_NAME
		
		self.playerName = ""
		self.ipAddr = ""
		
		# Play music
		self.resources.playMusic(MUSIC_MENU)
		
		# Hosting options
		self.condIdx = 0
		self.subCondIdx = 0
		
	# Main menu loop
	def doMenu(self):
		
		def message(s):
			self.resources.puttxt((512, 430), s, white4, center = True)
	
		def escTransit(state):
			if self.esc:
				self.selectIdx = 0
				self.select = False
				self.esc = False
				self.state = state
				return True
			return False				
			
		def selectTransit(lst):
			if self.select:
				sel = self.selectIdx % len(lst)
				self.select = False
				self.selectIdx = 0
				self.state = lst[sel]
				
		def selectList(lst, pos, center = True, step = 30):
			i = 0
			for l in lst:
				self.resources.puttxt((pos[0], pos[1]-i*step), l, (255, 255, ((self.selectIdx % len(lst)) != i)*255,255), center)
				i += 1
			
	
		for event in pygame.event.get():
			
			if event.type == MOUSEBUTTONDOWN:
				pos = convertScreenPos(event.pos)
			
			if event.type == KEYDOWN:
				if event.key == K_ESCAPE:
					self.esc = True
			
				if self.state == MENU_NAME:
					if K_a <= event.key and event.key <= K_z:
						self.playerName += str(chr(event.key - K_a + ord('a')))
					elif event.key == K_BACKSPACE and len(self.playerName):
						self.playerName = self.playerName[:-1]
						
				elif self.state == MENU_JOIN:
					if K_a <= event.key and event.key <= K_z:
						self.ipAddr += str(chr(event.key - K_a + ord('a')))
					elif K_0 <= event.key and event.key <= K_9:
						self.ipAddr += str(chr(event.key - K_0 + ord('0')))
					elif event.key == K_PERIOD:
						self.ipAddr += "."
					elif event.key == K_BACKSPACE and len(self.ipAddr):
						self.ipAddr = self.ipAddr[:-1]
						
				elif self.state == MENU_HOST:
					if event.key == K_c:
						self.condIdx = (self.condIdx + 1 ) % len(conditions)
						if self.condIdx == 1:
							self.subCondIdx = 0
					elif event.key == K_v:
						if self.condIdx == 0:
							self.subCondIdx = (self.subCondIdx + 1) % len(subconditions[0])
						else:
							self.subCondIdx = 0
						
				if event.key == K_w or event.key == K_UP:
					self.selectIdx -= 1
				elif event.key == K_s or event.key == K_DOWN:
					self.selectIdx += 1
					
				if event.key == K_RETURN or event.key == K_SPACE:
					self.select = True
	
		
		########################
		## QUIT GAME
		#########################
		if self.state == MENU_QUIT:
			return False
		
		######################
		## NAME INPUT
		#######################
		elif self.state == MENU_NAME:
			# Input name
			if escTransit(MENU_QUIT):
				return True
				
			selectTransit( [MENU_MAIN] )
			
			# Draw info
			message("Enter your nickname:")
			
			# Draw name
			self.resources.puttxt((512, 400), self.playerName, white4, center = True)
				
		######################
		## MAIN MENU
		#######################
		elif self.state == MENU_MAIN:
			# Display join and host option
			if escTransit(MENU_QUIT):
				return True
			
			selectList(["Join A Game", "Host A Game", "Quit"], (512, 420), True, 40)
				 
			selectTransit( [MENU_JOIN, MENU_HOST, MENU_QUIT] )
			
			# If not playing yet, start playing
			self.resources.playMusic(MUSIC_MENU)
				
		###################
		## HOST GAME MENU
		###################
		elif self.state == MENU_HOST:
			# Start hosting, display players
			# connected, and give option to
			# start a specified stage
			
			if not self.gamestate:
				gamestate.init()
				# Create Gamestate and server
				self.gamestate = gamestate.GGamestate(self.resources)
				# Create net system
				self.net = network.Net(self.gamestate, self.resources)
				
				## Add our player manually
				localPlayer = player.GPlayer(0, self.gamestate, self.playerName, self.gamestate.nextColor(), self.resources)
				self.gamestate.addPlayer(localPlayer, LOCAL_PLAYER)
				
				# Create server
				self.net.createServer()
				
				# Load the list of maps
				self.maps = os.listdir(MAP_PATH)
			else:
				# Receive incoming messages
				self.net.serverRead()
				
				message("Connected players:")
				
				# Display connected players
				i = 0
				for pl in self.gamestate.players:
					self.resources.puttxt((512, 380-i*20), pl.name, white4, center = True)
					i += 1
					
				# Put list of maps to chose from
				selectList(self.maps, (712, 320), False, 40)
				
				# Put win condition settings to the left
				self.resources.puttxt((20, 240), "Press C to cycle win condition", white4)
				self.resources.puttxt((60, 200), conditions[self.condIdx], white4)
				
				self.resources.puttxt((20, 140), "Press V to cycle subcondition", white4)
				self.resources.puttxt((60, 100), subconditions[self.condIdx][self.subCondIdx], white4)
				
				if self.select:
					sel = self.selectIdx
					self.select = False
					self.selectIdx = 0
					
					# Begin game
					self.resources.playMusic(MUSIC_GAME)
					
					if self.condIdx == 0:
						# score 
						points = 200*(1+self.subCondIdx)
					else:
						# means number of cars
						points = 5
					
					self.net.setupGame(self.maps[sel], points)

					self.gamestate.startGame()
					while self.gamestate.game:
						self.gamestate.doTime(self.net)
						
					# Game ended
					
					# Free Gamestate and Net
					self.net.disconnect()
					self.gamestate = False
					self.net = False
					
					self.state = MENU_MAIN

			
			if escTransit(MENU_MAIN):
				# Send server closed messages
				self.net.disconnect()
				self.gamestate = False
				self.net = False
				return True
			
		####################x
		### JOIN GAME MENU
		####################
		elif self.state == MENU_JOIN:
			# Give option to enter IP address
			if escTransit(MENU_MAIN):
				return True
			
			selectTransit( [MENU_JOINED] )
			
			# Draw info
			message("Enter the hostname:")
			
			# Draw name
			self.resources.puttxt((512, 400), self.ipAddr, (255, 255, 255,255), center = True)
			
			
		####################
		## JOINED STATE
		####################
		elif self.state == MENU_JOINED:
			# Display players connected
			# If update package arrives, begin game
			
			if escTransit(MENU_MAIN):
				self.net.disconnect()
				self.gamestate = False
				self.net = False
				return True
			
			if not self.gamestate:
				gamestate.init()
				self.gamestate = gamestate.GGamestate(self.resources)
				
				# Create net system
				self.net = network.Net(self.gamestate, self.resources)
				self.gamestate.eventhandler = self.net
				
				# Connect to the server
				if not self.net.connectToServer(self.playerName, self.ipAddr):
					self.esc = True
			else:
				# Check incoming
				self.net.clientRead()
			
				message("Waiting for reply.. Current players:")
					
				# Put player list
				i = 0
				for pl in self.gamestate.players:
					self.resources.puttxt((512, 380-i*20), pl.name, (255, 255, 255, 255), center = True)
					i += 1
					
				if self.net.update_pkg_count > 0:
					# The game begins
					self.resources.playMusic(MUSIC_GAME)
					self.gamestate.startGame()
					while self.gamestate.game:
						self.gamestate.doTime(self.net)
						
					# Free Gamestate and Net
					self.net.disconnect()
					self.gamestate = False
					self.net = False
					self.state = MENU_MAIN
				
				
	
		return True
		
		
	