from image import Image
from OpenGL.GL import *
from constants import *
import math

class Panel:
	def __init__(self):
		self.panel = [ (Image("panel/panel1" + str(i) + ".png", GL_NEAREST), (i*128, 0)) for i in range(0, 8) ]
		self.panel.extend( [ (Image("panel/panel0" + str(i) + ".png", GL_NEAREST), (i*128, 128)) for i in range(3,5)] )
		
		self.ap_hand = Image("ap_hand.png")
		self.speed_hand = Image("speed_hand.png")
		self.press_hand = Image("press_hand.png")
		self.press_hand_max = Image("press_hand_max.png")
		
		self.redlight = ( Image("redlight_off.png"), Image("redlight_on.png"))
		
		self.coal = ( Image("coal0.png"), Image("coal1.png"), Image("coal2.png"), Image("coal3.png"))

		self.ownplayer = None
			# Gamestate will register this later	
		
		self.coallist = []
		self.coalnum = 0
			# This will be compared to the actual coal
			# number. If differs, rebuild coal stack
	
	def render(self):
		#
		# Panel
		#
		for p in self.panel:
			p[0].draw(p[1])
		
		tr = self.ownplayer.train
		
		
		#
		# Coal
		# It's piled up in the bottom-left corner
		#
		
		if self.coalnum != tr.coal:
			# Rebuild list
			self.coalnum = tr.coal
			self.coallist = []
			i = 0
			m = COAL_PILE_SIZE
			prev_sumcoal = 0
			
			#
			# returns an index between (0..3), and a rotation
			# pseudo-rendomized by k
			#
			def alterCoal(k):
				rnd = k*k*0.367291
				return (int(math.floor(rnd)) % 4, rnd*180.0)
			
			while i < self.coalnum:
				while (i < prev_sumcoal + m) and i < self.coalnum:
					pos = ((i - prev_sumcoal)*COAL_X, (COAL_PILE_SIZE - m)*COAL_Y)
					self.coallist.insert(0, (pos, alterCoal(i)))
					i += 1
				prev_sumcoal += m
				m -= 1
		
		for c in self.coallist:
			self.coal[c[1][0]].drawCentered(c[0], rot = c[1][1])
		
		
		#
		# Red light
		# It is on, if the force is negative (eg. braking)
		#
		self.redlight[tr.force < 0].drawCentered((630, 80))
			
		#
		# AP hand canter: 899,3 top: 853,52 
		#
		glLoadIdentity()
		glTranslatef(899, 3, 0)
		glRotatef(133.0 - 90*self.ownplayer.action_points/MAX_ACTION_POINTS, 0.0, 0.0, 1.0)
		self.ap_hand.drawCentered((66,0), identity = False, rot = 220.0)
		
		
		#
		# Press hand: center 711,57
		#
		pos = (709, 59)
		self.press_hand.drawCentered((pos), rot = 24.0 - abs(tr.force)/MAX_FORCE*317)
		self.press_hand_max.drawCentered((pos), rot = 24.0 - tr.maxforce/MAX_FORCE*317)
		
		#
		# Speed hand: center 520, 111
		#
		self.speed_hand.drawCentered((520, 111), rot = -tr.v/150.0*320)