from engine import gamestate, player, render, net as network
from engine.constants import *

### DEBUG
import timing
import psyco

import pygame
from pygame.locals import *

from OpenGL.GL import *
from OpenGL.GLU import *

pygame.init()
pygame.display.set_mode((640, 480), OPENGL | HWSURFACE | DOUBLEBUF)

glClearColor(0.0, 0.0, 0.0, 0.0)
glClear(GL_COLOR_BUFFER_BIT)
glMatrixMode(GL_PROJECTION)
glOrtho(0, 639, 0, 479, -1, 1)
glMatrixMode(GL_MODELVIEW)


# Create blank gamestate
gamestate.init()
gs = gamestate.Gamestate()

# Create net system
net = network.Net(gs)

## Add our player manually
gs.addPlayer( player.Player(0, gs, "Ron", gs.nextColor() ))

# Create server
net.createServer()

psyco.profile()

# Wait for other players
while len(gs.players) < 2:
	net.serverRead()

# Setup Game
net.setupGame("proba.map", 2000)

# Everybody is here, align the trains (this is manual yet..)
ron = gs.players[0]
someone = gs.players[1]
#someone2 = gs.players[2]

ron.train.head = (gs.lines[0], 160, 1)
ron.train.v = 0.05
ron.train.alignToHead()

someone.train.head = (gs.lines[2], 80, 1)
someone.train.v = 0.04
someone.train.alignToHead()

#someone2.train.head = (gs.lines[1], 80, 1)
#someone2.train.v = 0.04
#someone2.train.alignToHead()

# Send Lobby End to signal start (maybe)
# ...

# Start riding

	
c = pygame.time.Clock()

c.tick()
currentTime = 0

# When to send next data update
nextUpdate = currentTime + NET_UPDATE_IVAL

while True:
	## Measure delta time
	delta_msec = c.tick()
	currentTime += delta_msec
	dt = delta_msec #/ 1000.0
	
	############################
	## Network phase one begins
	############################
	
	## Check incoming events
	net.serverRead()
	
	############################
	##### Modelling stage begins
	############################
	
	## Switch refresh
	for sw in gs.switches:
		sw.beginRound()
		sw.refresh()
	
	## Move trains
	for pl in gs.players:
		pl.train.move(dt)
		
	gs.checkCollision()
	
	## Switch refresh end phase
	for sw in gs.switches:
		sw.endRound()
			
	##########################
	## Network update phase
	##########################
	
	if currentTime >= nextUpdate:
		# Update required
		net.sendUpdate()
		# Schedule next update
		nextUpdate = currentTime + NET_UPDATE_IVAL
	
	###########################
	## Drawing phase
	###########################
	#timing.start()
	glClear(GL_COLOR_BUFFER_BIT)
	
	for ln in gs.lines:
		render.renderLineBase(ln)
		render.renderLineTop(ln)
	
	#for ln in gs.lines:
	#	render.renderLineTop(ln)
		
	gs.render()
	glFlush()
	pygame.display.flip()
	
	#timing.finish()
	#print 'Draw:', timing.milli()
	
	for event in pygame.event.get():
		if event.type == KEYDOWN:
			if event.key == K_s:
				gs.switches[1].bidForState(ron, 0)
				#gs.switches[1].bid.topstate = 1 - gs.switches[1].bid.topstate
				#print 'switching sw#1 to', gs.switches[1].bid.topstate
			
			if event.key == K_a:
				ron.train.addCar(1)
				#gs.switches[1].bidForState(ron, 1)
				
				#gs.switches[0].bid.topstate = 1 - gs.switches[0].bid.topstate
				#print 'switching sw#0 to', gs.switches[0].bid.topstate
	
	################
	## Sleep phase
	################	
	if currentTime % 1000 < 20:
		fps_div = delta_msec
		if fps_div == 0:
			fps_div = 1000
		print 'FPS', 1000/fps_div
	pygame.time.wait(5)
